@@ -1,10 +1,9 @@
 /*-*-c++-*-
- * $Id: kpsion.cpp,v 1.13 2001/05/22 21:59:09 felfert Exp $
+ * $Id: kpsion.cpp,v 1.21 2003/02/03 08:29:25 felfert Exp $
  *
  * This file is part of plptools.
  *
- *  Copyright (C) 1999  Philip Proudman <philip.proudman@btinternet.com>
- *  Copyright (C) 2000, 2001 Fritz Elfert <felfert@to.com>
+ *  Copyright (C) 2000-2002 Fritz Elfert <felfert@to.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -29,7 +28,6 @@
 #include "kpsionconfig.h"
 #include "wizards.h"
 
-#include <kapp.h>
 #include <klocale.h>
 #include <kaction.h>
 #include <kstdaction.h>
@@ -38,6 +36,8 @@
 #include <kmessagebox.h>
 #include <kfileitem.h>
 #include <kprocess.h>
+#include <kstatusbar.h>
+#include <kiconloader.h>
 
 #include <qwhatsthis.h>
 #include <qtimer.h>
@@ -46,11 +46,14 @@
 #include <qheader.h>
 #include <qdir.h>
 #include <qmessagebox.h>
+#include <qregexp.h>
+#include <qstyle.h>
 
 #include <ppsocket.h>
 #include <rfsvfactory.h>
 #include <rpcsfactory.h>
 #include <bufferarray.h>
+#include <psiprocess.h>
 
 #define STID_CONNECTION 1
 
@@ -176,7 +179,11 @@ psion2unix(const char * const path) {
     QString tmp = path;
     tmp.replace(QRegExp("%"), "%25");
     tmp.replace(QRegExp("/"), "%2f");
+#if QT_VERSION >= 300
+    tmp.replace(QRegExp("\\\\"), "/");
+#else
     tmp.replace(QRegExp("\\"), "/");
+#endif
     return tmp;
 }
 
@@ -292,15 +299,84 @@ queryPsion() {
 
     statusBar()->changeItem(i18n("Retrieving machine info ..."),
 			    STID_CONNECTION);
-    rpcs::machineInfo mi;
-    if ((res = plpRpcs->getMachineInfo(mi)) != rfsv::E_PSI_GEN_NONE) {
-	QString msg = i18n("Could not get Psion machine info");
+
+    Enum<rpcs::machs> machType;
+    if (plpRpcs->getMachineType(machType) != rfsv::E_PSI_GEN_NONE) {
+	QString msg = i18n("Could not get Psion machine type");
 	statusBar()->changeItem(msg, STID_CONNECTION);
 	KMessageBox::error(this, msg);
 	return;
     }
-    machineUID = mi.machineUID;
-    S5mx = (strcmp(mi.machineName, "SERIES5mx") == 0);
+    if (machType == rpcs::PSI_MACH_S5) {
+	rpcs::machineInfo mi;
+	if ((res = plpRpcs->getMachineInfo(mi)) != rfsv::E_PSI_GEN_NONE) {
+	    QString msg = i18n("Could not get Psion machine info");
+	    statusBar()->changeItem(msg, STID_CONNECTION);
+	    KMessageBox::error(this, msg);
+	    return;
+	}
+	machineUID = mi.machineUID;
+	S5mx = (strcmp(mi.machineName, "SERIES5mx") == 0);
+    } else {
+	// On a SIBO, first check for a file 'SYS$PT.CFG' on the default
+	// directory. If it exists, read the UID from it. Otherwise
+	// calculate a virtual machine UID from the OwnerInfo data and
+	// write it to that file.
+	bufferArray b;
+	u_int32_t handle;
+	u_int32_t count;
+
+	res = plpRfsv->fopen(plpRfsv->opMode(rfsv::PSI_O_RDONLY),
+			     "SYS$PT.CFG", handle);
+	if (res == rfsv::E_PSI_GEN_NONE) {
+	    res = plpRfsv->fread(handle, (unsigned char *)&machineUID,
+				 sizeof(machineUID), count);
+	    plpRfsv->fclose(handle);
+	    if ((res != rfsv::E_PSI_GEN_NONE) || (count != sizeof(machineUID))) {
+		QString msg = i18n("Could not read SIBO UID file");
+		statusBar()->changeItem(msg, STID_CONNECTION);
+		KMessageBox::error(this, msg);
+		return;
+	    }
+	} else {
+	    if ((res = plpRpcs->getOwnerInfo(b)) != rfsv::E_PSI_GEN_NONE) {
+		QString msg = i18n("Could not get Psion owner info");
+		statusBar()->changeItem(msg, STID_CONNECTION);
+		KMessageBox::error(this, msg);
+		return;
+	    }
+	    machineUID = 0;
+	    std::string oi = "";
+	    while (!b.empty()) {
+		oi += b.pop().getString();
+		oi += "\n";
+	    }
+	    const char *p = oi.c_str();
+	    unsigned long long z;
+	    int i = 0;
+
+	    while (*p) {
+                z = *p;
+                machineUID ^= (z << i);
+                p++; i++;
+                i &= ((sizeof(machineUID) * 8) - 1);
+	    }
+	    res = plpRfsv->fcreatefile(plpRfsv->opMode(rfsv::PSI_O_RDWR),
+				 "SYS$PT.CFG", handle);
+	    if (res == rfsv::E_PSI_GEN_NONE) {
+		res = plpRfsv->fwrite(handle, (const unsigned char *)&machineUID,
+				      sizeof(machineUID), count);
+		plpRfsv->fclose(handle);
+	    }
+	    if (res != rfsv::E_PSI_GEN_NONE) {
+		QString msg = i18n("Could not write SIBO UID file %1").arg((const char *)res);
+		statusBar()->changeItem(msg, STID_CONNECTION);
+		KMessageBox::error(this, msg);
+		return;
+	    }
+        }
+	S5mx = false;
+    }
 
     QString uid = getMachineUID();
     bool machineFound = false;
@@ -443,13 +519,23 @@ slotAutoAction() {
 	QCStringList::Iterator it;
 
 	for (it = argl.begin(); it != argl.end(); ++it) {
-	    QString drv((*it).upper());
-
-	    for (i = view->firstItem(); i; i = i->nextItem()) {
-		if (i->key() == drv) {
-		    i->setSelected(true);
-		    any = true;
+	    i = 0L;
+	    if ((*it).left(7) == "psion:/") {
+		QString volname((*it).mid(7));
+		for (i = view->firstItem(); i; i = i->nextItem()) {
+		    if (i->text().startsWith(volname + " "))
+			break;
 		}
+	    } else {
+		QString drv((*it).upper());
+		for (i = view->firstItem(); i; i = i->nextItem()) {
+		    if (i->key() == drv)
+			break;
+		}
+	    }
+	    if (i) {
+		i->setSelected(true);
+		any = true;
 	    }
 	}
 	if (any) {
@@ -467,18 +553,29 @@ slotAutoAction() {
 	QCStringList::Iterator it;
 
 	for (it = argl.begin(); it != argl.end(); ++it) {
-	    QString drv((*it).upper());
-	    if (drv == "Z") {
-		KMessageBox::sorry(this, i18n(
-		    "<QT>The selected drive <B>Z:</B> is "
-		    "a <B>ROM</B> drive and therefore cannot be restored.</QT>"));
-		continue;
-	    }
-	    for (i = view->firstItem(); i; i = i->nextItem()) {
-		if (i->key() == drv) {
-		    i->setSelected(true);
-		    any = true;
+	    i = 0L;
+	    if ((*it).left(7) == "psion:/") {
+		QString volname((*it).mid(7));
+		for (i = view->firstItem(); i; i = i->nextItem()) {
+		    if (i->text().startsWith(volname + " "))
+			break;
 		}
+	    } else {
+		QString drv((*it).upper());
+		for (i = view->firstItem(); i; i = i->nextItem()) {
+		    if (i->key() == drv)
+			break;
+		}
+	    }
+	    if (i) {
+		if (i->key() == "Z") {
+		    KMessageBox::sorry(this, i18n(
+			"<QT>The selected drive <B>Z:</B> is "
+			"a <B>ROM</B> drive and therefore cannot be restored.</QT>"));
+		    continue;
+		}
+		i->setSelected(true);
+		any = true;
 	    }
 	}
 	if (any)
@@ -493,12 +590,29 @@ slotAutoAction() {
 	QCStringList::Iterator it;
 
 	for (it = argl.begin(); it != argl.end(); ++it) {
-	    QString drv((*it).upper());
-	    for (i = view->firstItem(); i; i = i->nextItem()) {
-		if (i->key() == drv) {
-		    i->setSelected(true);
-		    any = true;
+	    i = 0L;
+	    if ((*it).left(7) == "psion:/") {
+		QString volname((*it).mid(7));
+		for (i = view->firstItem(); i; i = i->nextItem()) {
+		    if (i->text().startsWith(volname + " "))
+			break;
 		}
+	    } else {
+		QString drv((*it).upper());
+		for (i = view->firstItem(); i; i = i->nextItem()) {
+		    if (i->key() == drv)
+			break;
+		}
+	    }
+	    if (i) {
+		if (i->key() == "Z") {
+		    KMessageBox::sorry(this, i18n(
+			"<QT>The selected drive <B>Z:</B> is "
+			"a <B>ROM</B> drive and therefore cannot be formatted.</QT>"));
+		    continue;
+		}
+		i->setSelected(true);
+		any = true;
 	    }
 	}
 	if (any)
@@ -540,20 +654,28 @@ startupNcpd() {
 	return;
     KProcess proc;
     ppsocket *testSocket;
+    time_t start_time = time(0L) + 2;
+    bool connectOk = false;
 
     testSocket = new ppsocket();
     if (!testSocket->connect(NULL, 7501)) {
-	time_t start_time = time(0L) + 2;
 
 	statusBar()->changeItem(i18n("Starting ncpd daemon ..."),
 				STID_CONNECTION);
 	proc << ncpdPath;
 	proc << "-s" << ncpdDevice << "-b" << ncpdSpeed;
 	proc.start(KProcess::DontCare);
-	while ((time(0L) < start_time) && (!testSocket->connect(NULL, 7501)))
+	while ((time(0L) < start_time) &&
+	       (!(connectOk = testSocket->connect(NULL, 7501))))
 	    kapp->processEvents();
     }
     delete testSocket;
+    if (connectOk) {
+	// 2 more seconds for ncpd to negotiate with the Psion.
+	start_time = time(0L) + 2;
+	while (time(0L) < start_time)
+	    kapp->processEvents();
+    }
 }
 
 void KPsionMainWindow::
@@ -828,7 +950,11 @@ doBackup() {
     strftime(tstr, sizeof(tstr), "%Y-%m-%d-%H-%M-%S.tmp.gz",
 	     localtime(&now));
     archiveName += tstr;
+#if KDE_VERSION >= 300
+    backupTgz = new KTar(archiveName, "application/x-gzip");
+#else
     backupTgz = new KTarGz(archiveName);
+#endif
     backupTgz->open(IO_WriteOnly);
     createIndex();
 
@@ -843,7 +969,6 @@ doBackup() {
     for (int i = 0; i < toBackup.size(); i++) {
 	PlpDirent e = toBackup[i];
 	const char *fn = e.getName();
-	QString unixname = psion2unix(fn);
 	QByteArray ba;
 	QDataStream os(ba, IO_WriteOnly);
 
@@ -856,38 +981,51 @@ doBackup() {
 	u_int32_t handle;
 
 	kapp->processEvents();
-	res = plpRfsv->fopen(plpRfsv->opMode(rfsv::PSI_O_RDONLY), fn,
-			     handle);
-	if (res != rfsv::E_PSI_GEN_NONE) {
-	    if (KMessageBox::warningYesNo(this, i18n("<QT>Could not open<BR/><B>%1</B></QT>").arg(fn)) == KMessageBox::No) {
-		badBackup = true;
-		break;
-	    } else {
-		e.setName("!");
-		continue;
-	    }
-	}
-	unsigned char *buff = new unsigned char[RFSV_SENDLEN];
-	u_int32_t len;
+	bool tryLoop = true;
 	do {
-	    if ((res = plpRfsv->fread(handle, buff, RFSV_SENDLEN, len)) ==
-		rfsv::E_PSI_GEN_NONE) {
-		os.writeRawBytes((char *)buff, len);
-		updateProgress(len);
+	    res = plpRfsv->fopen(plpRfsv->opMode(rfsv::PSI_O_RDONLY), fn,
+				 handle);
+	    if (res == rfsv::E_PSI_GEN_NONE) {
+		unsigned char *buff = new unsigned char[RFSV_SENDLEN];
+		u_int32_t len;
+		do {
+		    if ((res = plpRfsv->fread(handle, buff, RFSV_SENDLEN,
+					      len)) == rfsv::E_PSI_GEN_NONE) {
+			os.writeRawBytes((char *)buff, len);
+			updateProgress(len);
+		    }
+		} while ((len > 0) && (res == rfsv::E_PSI_GEN_NONE));
+		delete[]buff;
+		plpRfsv->fclose(handle);
 	    }
-	} while ((len > 0) && (res == rfsv::E_PSI_GEN_NONE));
-	delete[]buff;
-	plpRfsv->fclose(handle);
-	if (res != rfsv::E_PSI_GEN_NONE) {
-	    if (KMessageBox::warningYesNo(this, i18n("<QT>Could not read<BR/><B>%1</B></QT>").arg(fn)) == KMessageBox::No) {
-		badBackup = true;
-		break;
+	    if (res != rfsv::E_PSI_GEN_NONE) {
+		switch (KMessageBox::warningYesNoCancel(
+			    this, i18n(
+				"<QT>Could not backup<BR/><B>%1</B><BR/>"
+				"<FONT COLOR=RED>%2</FONT><BR/></QT>"
+				).arg(fn).arg((const char *)res),
+			    QString::null, i18n("Retry"), i18n("Ignore"))) {
+		    case KMessageBox::Cancel:
+			badBackup = true;
+			tryLoop = false;
+			break;
+		    case KMessageBox::No:
+			e.setName("!");
+			tryLoop = false;
+			break;
+		    case KMessageBox::Yes:
+			break;
+		}
 	    } else {
-		e.setName("!");
-		continue;
+		tryLoop = false;
 	    }
-	}
-	backupTgz->writeFile(unixname, "root", "root", ba.size(), ba.data());
+	} while (tryLoop);
+	if (badBackup)
+	    break;
+	if (res != rfsv::E_PSI_GEN_NONE)
+	    continue;
+	backupTgz->writeFile(psion2unix(fn), "root", "root", ba.size(),
+			     ba.data());
     }
 
     if (!badBackup) {
@@ -907,7 +1045,7 @@ doBackup() {
 		kapp->processEvents();
 		res = plpRfsv->fsetattr(fn, 0, rfsv::PSI_A_ARCHIVE);
 		if (res != rfsv::E_PSI_GEN_NONE) {
-		    if (KMessageBox::warningYesNo(this, i18n("<QT>Could not set attributes of<BR/><B>%1</B></QT>").arg(fn)) == KMessageBox::No) {
+		    if (KMessageBox::warningYesNo(this, i18n("<QT>Could not set attributes of<BR/><B>%1</B><BR/><FONT COLOR=red>%2</FONT><BR/>Continue?</QT>").arg(fn)) == KMessageBox::No) {
 			break;
 		    }
 		}
@@ -990,13 +1128,16 @@ removeOldBackups(QStringList &drives) {
     QFileInfoListIterator it(*fil);
     QFileInfo *fi;
     ArchList alist;
-    Barchive *a;
 
     // Build a list of full-backups sorted by date
     while ((fi = it.current())) {
 	kapp->processEvents();
 
+#if KDE_VERSION >= 300
+	KTar tgz(fi->absFilePath());
+#else
 	KTarGz tgz(fi->absFilePath());
+#endif
 	const KTarEntry *te;
 
 	tgz.open(IO_ReadOnly);
@@ -1036,7 +1177,11 @@ removeOldBackups(QStringList &drives) {
     while ((fi = it.current())) {
 	kapp->processEvents();
 
+#if KDE_VERSION >= 300
+	KTar tgz(fi->absFilePath());
+#else
 	KTarGz tgz(fi->absFilePath());
+#endif
 	const KTarEntry *te;
 	bool valid = false;
 	bool del = false;
@@ -1115,8 +1260,14 @@ askOverwrite(PlpDirent e) {
 
     lay->addStretch(1);
     QLabel *label1 = new QLabel(contents);
+
+#if (QT_VERSION >= 300)
+    label1->setPixmap(kapp->style().stylePixmap(
+			  kapp->style().SP_MessageBoxWarning));
+#else
     label1->setPixmap(QMessageBox::standardIcon(QMessageBox::Warning,
 						kapp->style().guiStyle()));
+#endif
     lay->add(label1);
     lay->add(new QLabel(i18n(
 	"<QT>The file <B>%1</B> exists already on the Psion with "
@@ -1210,7 +1361,11 @@ slotStartRestore() {
 	for (t = tars.begin(); t != tars.end(); t++) {
 	    PlpDir toRestore = restoreDialog.getRestoreList(*t);
 	    if (toRestore.size() > 0) {
+#if KDE_VERSION >= 300
+		KTar tgz(*t);
+#else
 		KTarGz tgz(*t);
+#endif
 		const KTarEntry *te;
 		QString pDir("");
 
@@ -1220,7 +1375,6 @@ slotStartRestore() {
 		    PlpDirent olde;
 
 		    const char *fn = e.getName();
-		    QString unixname = psion2unix(fn);
 		    Enum<rfsv::errs> res;
 
 		    progressLocal = e.getSize();
@@ -1229,7 +1383,7 @@ slotStartRestore() {
 		    emit setProgressText(QString("%1").arg(fn));
 		    emit setProgress(0);
 
-		    te = findTarEntry(tgz.directory(), unixname);
+		    te = findTarEntry(tgz.directory(), psion2unix(fn));
 		    if (te != 0L) {
 			u_int32_t handle;
 			QString cpDir(fn);
@@ -1294,6 +1448,7 @@ slotStartRestore() {
 			    updateProgress(written);
 			} while (len > 0);
 			plpRfsv->fclose(handle);
+
 			if (res != rfsv::E_PSI_GEN_NONE) {
 			    KMessageBox::error(this, i18n(
 				"<QT>Could not write to<BR/>"
@@ -1584,33 +1739,18 @@ collectFiles(QString dir) {
 void KPsionMainWindow::
 killSave() {
     Enum<rfsv::errs> res;
-    bufferArray tmp;
+    processList tmp;
 
     savedCommands.clear();
-    if ((res = plpRpcs->queryDrive('C', tmp)) != rfsv::E_PSI_GEN_NONE) {
+    if ((res = plpRpcs->queryPrograms(tmp)) != rfsv::E_PSI_GEN_NONE) {
 	cerr << "Could not get process list, Error: " << res << endl;
 	return;
     } else {
-	while (!tmp.empty()) {
-	    QString pbuf;
-	    bufferStore cmdargs;
-	    bufferStore bs = tmp.pop();
-	    int pid = bs.getWord(0);
-	    const char *proc = bs.getString(2);
-	    if (S5mx)
-		pbuf.sprintf("%s.$%02d", proc, pid);
-	    else
-		pbuf.sprintf("%s.$%d", proc, pid);
-	    bs = tmp.pop();
-	    if (plpRpcs->getCmdLine(pbuf.data(), cmdargs) == 0) {
-		QString cmdline(cmdargs.getString(0));
-		cmdline += " ";
-		cmdline += bs.getString(0);
-		savedCommands += cmdline;
-	    }
-	    emit setProgressText(i18n("Stopping %1").arg(cmdargs.getString(0)));
+	for (processList::iterator i = tmp.begin(); i != tmp.end(); i++) {
+	    savedCommands += i->getArgs();
+	    emit setProgressText(i18n("Stopping %1").arg(i->getName()));
 	    kapp->processEvents();
-	    plpRpcs->stopProgram(pbuf);
+	    plpRpcs->stopProgram(i->getProcId());
 	}
     }
     time_t tstart = time(0) + 5;
@@ -1618,7 +1758,7 @@ killSave() {
 	kapp->processEvents();
 	usleep(100000);
 	kapp->processEvents();
-	if ((res = plpRpcs->queryDrive('C', tmp)) != rfsv::E_PSI_GEN_NONE) {
+	if ((res = plpRpcs->queryPrograms(tmp)) != rfsv::E_PSI_GEN_NONE) {
 	    cerr << "Could not get process list, Error: " << res << endl;
 	    return;
 	}
@@ -1629,7 +1769,7 @@ killSave() {
 		"<QT>Could not stop all processes.<BR/>"
 		"Please stop running programs manually on the Psion, "
 		"then klick <B>Ok</B>."));
-	    time_t tstart = time(0) + 5;
+	    tstart = time(0) + 5;
 	}
     }
     return;
@@ -1693,6 +1833,7 @@ createIndex() {
 	PsiTime t = e.getPsiTime();
 	long attr = e.getAttr() &
 	    ~rfsv::PSI_A_ARCHIVE;
+#if (QT_VERSION < 300)
 	os << hex
 	   << setw(8) << setfill('0') <<
 	    t.getPsiTimeHi() << " "
@@ -1703,6 +1844,18 @@ createIndex() {
 	   << setw(8) << setfill('0') <<
 	    attr << " "
 	   << setw(0) << e.getName() << endl;
+#else
+	os << hex
+	   << qSetW(8) << qSetFill('0') <<
+	    t.getPsiTimeHi() << " "
+	   << qSetW(8) << qSetFill('0') <<
+	    t.getPsiTimeLo() << " "
+	   << qSetW(8) << qSetFill('0') <<
+	    e.getSize() << " "
+	   << qSetW(8) << qSetFill('0') <<
+	    attr << " "
+	   << qSetW(0) << e.getName() << endl;
+#endif
 	kapp->processEvents();
     }
     QString idxName =
