@@ -1,9 +1,9 @@
 /*-*-c++-*-
- * $Id: plpprops.cc,v 1.5 2001/03/28 12:17:37 felfert Exp $
+ * $Id: plpprops.cc,v 1.10 2002/07/14 20:12:37 felfert Exp $
  *
  * This file is part of plptools.
  *
- *  Copyright (C) 2001 Fritz Elfert <felfert@to.com>
+ *  Copyright (C) 2001-2002 Fritz Elfert <felfert@to.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -38,13 +38,25 @@
 #include <qtabwidget.h>
 #include <qcheckbox.h>
 #include <qwhatsthis.h>
-
-#include <rfsv.h>
-#include <rpcs.h>
+#include <qpushbutton.h>
 
 #include <strstream>
 #include <iomanip>
 
+#ifdef ENABLE_NLS
+#define _PLPINTL_H_ // Override NLS stuff in headers from libplp
+static inline QString X_(const char *t) {
+    return KGlobal::locale()->translate(t);
+}
+static inline QString _(const char *t) {
+    return KGlobal::locale()->translate(t);
+}
+static inline const char *N_(const char *t) { return t; }
+#endif
+
+#include <rfsv.h>
+#include <rpcs.h>
+
 #define PLP_CMD_DRIVEINFO 1
 #define PLP_CMD_OWNERINFO 2
 #define PLP_CMD_GETATTR   3
@@ -54,6 +66,8 @@
 #define KIO_ARGS QByteArray packedArgs; \
 QDataStream stream( packedArgs, IO_WriteOnly ); stream
 
+using namespace std;
+
 class PlpPropsPlugin::PlpPropsPluginPrivate {
 public:
     PlpPropsPluginPrivate() { }
@@ -332,8 +346,9 @@ PlpFileAttrPage::PlpFileAttrPage(KPropertiesDialog *_props) {
 
     d->jobReturned = false;
     KIO_ARGS << int(PLP_CMD_GETATTR) << _props->item()->url().path();
-    KIO::StatJob *job = new KIO::StatJob(KURL("psion:/"), KIO::CMD_SPECIAL,
-					 packedArgs, false);
+    KURL u(_props->item()->url());
+    u.setPath("/");
+    KIO::StatJob *job = new KIO::StatJob(u, KIO::CMD_SPECIAL, packedArgs, false);
     connect(job, SIGNAL(result(KIO::Job *)),
 	    SLOT(slotGetSpecialFinished(KIO::Job *)));
 }
@@ -389,9 +404,10 @@ void PlpFileAttrPage::applyChanges() {
 
 	KIO_ARGS << int(PLP_CMD_SETATTR) << sattr << dattr
 		 << d->props->item()->url().path();
-	KIO::SimpleJob *sjob = new KIO::SimpleJob(KURL("psion:/"),
-						  KIO::CMD_SPECIAL, packedArgs,
-						  false);
+	KURL u(d->props->item()->url());
+	u.setPath("/");
+	KIO::SimpleJob *sjob = new KIO::SimpleJob(u, KIO::CMD_SPECIAL,
+						  packedArgs, false);
 	connect(sjob, SIGNAL(result(KIO::Job *)),
 		SLOT(slotSetSpecialFinished(KIO::Job *)));
     }
@@ -583,8 +599,9 @@ PlpDriveAttrPage::PlpDriveAttrPage(KPropertiesDialog *_props) {
     box->addStretch(10);
 
     KIO_ARGS << int(PLP_CMD_DRIVEINFO) << _props->item()->url().path();
-    KIO::StatJob *job = new KIO::StatJob(KURL("psion:/"), KIO::CMD_SPECIAL,
-					 packedArgs, false);
+    KURL u(_props->item()->url());
+    u.setPath("/");
+    KIO::StatJob *job = new KIO::StatJob(u, KIO::CMD_SPECIAL, packedArgs, false);
     connect(job, SIGNAL(result(KIO::Job *)),
 	    SLOT(slotSpecialFinished(KIO::Job *)));
 
@@ -713,10 +730,12 @@ public:
     QLabel *mbattMaxVoltage;
 
     QLabel *bbattStatus;
-    QLabel *bbattUsage;
     QLabel *bbattVoltage;
     QLabel *bbattMaxVoltage;
 
+    QLabel *epowerSupplied;
+    QLabel *epowerUsage;
+
     rpcs::machineInfo mi;
 };
 
@@ -740,7 +759,7 @@ PlpMachinePage::PlpMachinePage( KPropertiesDialog *_props ) {
     box = new QVBoxLayout(d->f, KDialog::spacingHint());
     gb = new QGroupBox(i18n("General"), d->f, "genInfBox");
     box->addWidget(gb);
-    d->g = new QGridLayout(gb, 1, 1, KDialog::marginHint(),
+    d->g = new QGridLayout(gb, 8, 2, KDialog::marginHint(),
 			   KDialog::spacingHint());
 
     d->machType = makeEntry(i18n("Machine type:"), gb, 1);
@@ -767,7 +786,7 @@ PlpMachinePage::PlpMachinePage( KPropertiesDialog *_props ) {
 
     gb = new QGroupBox(i18n("Time"), d->f, "timeInfBox");
     box->addWidget(gb);
-    d->g = new QGridLayout(gb, 1, 1, KDialog::marginHint(),
+    d->g = new QGridLayout(gb, 5, 2, KDialog::marginHint(),
 			   KDialog::spacingHint());
     d->machTime = makeEntry(i18n("Date/Time:"), gb, 1);
     QWhatsThis::add(d->machTime,
@@ -787,7 +806,7 @@ PlpMachinePage::PlpMachinePage( KPropertiesDialog *_props ) {
     box = new QVBoxLayout(d->f, KDialog::spacingHint());
     gb = new QGroupBox(i18n("Main battery"), d->f, "mbatInfBox");
     box->addWidget(gb);
-    d->g = new QGridLayout(gb, 1, 1, KDialog::marginHint(),
+    d->g = new QGridLayout(gb, 9, 2, KDialog::marginHint(),
 			   KDialog::spacingHint());
     d->mbattChanged    = makeEntry(i18n("Changed at:"), gb, 1);
     QWhatsThis::add(d->mbattChanged,
@@ -816,23 +835,34 @@ PlpMachinePage::PlpMachinePage( KPropertiesDialog *_props ) {
 
     gb = new QGroupBox(i18n("Backup battery"), d->f, "bbatInfBox");
     box->addWidget(gb);
-    d->g = new QGridLayout(gb, 1, 1, KDialog::marginHint(),
+    d->g = new QGridLayout(gb, 5, 2, KDialog::marginHint(),
 			   KDialog::spacingHint());
-    d->bbattUsage      = makeEntry(i18n("Usage time:"), gb, 1);
-    QWhatsThis::add(d->bbattUsage,
-		    i18n("This shows the accumulated time of running on backup battery power."));
-    d->bbattStatus     = makeEntry(i18n("Status:"), gb, 2);
+    d->bbattStatus     = makeEntry(i18n("Status:"), gb, 1);
     QWhatsThis::add(d->bbattStatus,
 		    i18n("This shows current status of the backup battery."));
-    d->bbattVoltage    = makeEntry(i18n("Voltage:"), gb, 3);
+    d->bbattVoltage    = makeEntry(i18n("Voltage:"), gb, 2);
     QWhatsThis::add(d->bbattVoltage,
 		    i18n("This shows the current backup battery voltage."));
-    d->bbattMaxVoltage = makeEntry(i18n("Max. voltage:"), gb, 4);
+    d->bbattMaxVoltage = makeEntry(i18n("Max. voltage:"), gb, 3);
     QWhatsThis::add(d->bbattMaxVoltage,
 		    i18n("This shows the maximum backup battery voltage."));
     d->g->addRowSpacing(0, KDialog::marginHint());
     d->g->setColStretch(0, 1);
     d->g->setColStretch(1, 1);
+
+    gb = new QGroupBox(i18n("External power"), d->f, "epowerInfBox");
+    box->addWidget(gb);
+    d->g = new QGridLayout(gb, 4, 2, KDialog::marginHint(),
+			   KDialog::spacingHint());
+    d->epowerSupplied   = makeEntry(i18n("Supplied:"), gb, 1);
+    QWhatsThis::add(d->epowerSupplied,
+		    i18n("This shows whether external power is currently supplied."));
+    d->epowerUsage      = makeEntry(i18n("Usage time:"), gb, 2);
+    QWhatsThis::add(d->epowerUsage,
+		    i18n("This shows the accumulated time of running on external power."));
+    d->g->addRowSpacing(0, KDialog::marginHint());
+    d->g->setColStretch(0, 1);
+    d->g->setColStretch(1, 1);
     box->addStretch(10);
 
     d->f = _props->dialog()->addPage(i18n("Psion M&emory"));
@@ -840,7 +870,7 @@ PlpMachinePage::PlpMachinePage( KPropertiesDialog *_props ) {
     box = new QVBoxLayout(d->f, KDialog::spacingHint());
     gb = new QGroupBox(i18n("ROM"), d->f, "romInfBox");
     box->addWidget(gb);
-    d->g = new QGridLayout(gb, 1, 1, KDialog::marginHint(),
+    d->g = new QGridLayout(gb, 5, 2, KDialog::marginHint(),
 			   KDialog::spacingHint());
     d->romVersion = makeEntry(i18n("Version:"), gb, 1);
     QWhatsThis::add(d->romVersion,
@@ -857,7 +887,7 @@ PlpMachinePage::PlpMachinePage( KPropertiesDialog *_props ) {
 
     gb = new QGroupBox(i18n("RAM"), d->f, "ramInfBox");
     box->addWidget(gb);
-    d->g = new QGridLayout(gb, 1, 1, KDialog::marginHint(),
+    d->g = new QGridLayout(gb, 6, 2, KDialog::marginHint(),
 			   KDialog::spacingHint());
     d->ramSize    = makeEntry(i18n("Size:"), gb, 1);
     QWhatsThis::add(d->ramSize,
@@ -876,9 +906,8 @@ PlpMachinePage::PlpMachinePage( KPropertiesDialog *_props ) {
     d->g->setColStretch(1, 1);
     box->addStretch(10);
 
-//    KIO_ARGS << int(PLP_CMD_MACHINFO);
-//    PlpMachInfoJob *job = new PlpMachInfoJob(packedArgs);
-    KURL u("psion:/0:_MachInfo");
+    KURL u(_props->item()->url());
+    u.setPath("/0:_MachInfo");
     KIO::TransferJob *job = KIO::get(u, false, false);
 
     connect(job, SIGNAL(result(KIO::Job *)),
@@ -902,10 +931,8 @@ bool PlpMachinePage::supports(KFileItemList _items) {
 }
 
 void PlpMachinePage::slotJobData(KIO::Job *job, const QByteArray &data) {
-    cout << "Mjdata" << endl;
     if (data.size() == sizeof(d->mi)) {
 	memcpy((char *)&d->mi, data, sizeof(d->mi));
-	cout << "got machInfo" << endl;
 
 	d->machType->setText(KGlobal::locale()->translate(d->mi.machineType));
 	d->machName->setText(QString(d->mi.machineName));
@@ -914,20 +941,35 @@ void PlpMachinePage::slotJobData(KIO::Job *job, const QByteArray &data) {
 	s << hex << setw(16) << d->mi.machineUID << '\0';
 	d->machUID->setText(QString(s.str()));
 	d->machLang->setText(KGlobal::locale()->translate(d->mi.uiLanguage));
-	d->dispGeo->setText(QString("%1x%2").arg(d->mi.displayWidth).arg(d->mi.displayHeight));
-	d->regSize->setText(QString("%1 (%2)").arg(KIO::convertSize(d->mi.registrySize)).arg(KGlobal::locale()->formatNumber(d->mi.registrySize, 0)));
+	d->dispGeo->setText(QString("%1x%2").arg(
+				d->mi.displayWidth).arg(d->mi.displayHeight));
+	d->regSize->setText(
+	    QString("%1 (%2)").arg(
+		KIO::convertSize(d->mi.registrySize)).arg(
+		    KGlobal::locale()->formatNumber(d->mi.registrySize, 0)));
 
 	QString rev;
 	rev.sprintf("%d.%02d(%d)", d->mi.romMajor, d->mi.romMinor,
 		    d->mi.romBuild);
 	d->romVersion->setText(rev);
-	d->romSize->setText(QString("%1 (%2)").arg(KIO::convertSize(d->mi.romSize)).arg(KGlobal::locale()->formatNumber(d->mi.romSize, 0)));
+	d->romSize->setText(
+	    QString("%1 (%2)").arg(KIO::convertSize(d->mi.romSize)).arg(
+		KGlobal::locale()->formatNumber(d->mi.romSize, 0)));
 	d->romProg->setText(d->mi.romProgrammable ? i18n("yes") : i18n("no"));
 
-	d->ramSize->setText(QString("%1 (%2)").arg(KIO::convertSize(d->mi.ramSize)).arg(KGlobal::locale()->formatNumber(d->mi.ramSize, 0)));
-	d->ramFree->setText(QString("%1 (%2)").arg(KIO::convertSize(d->mi.ramFree)).arg(KGlobal::locale()->formatNumber(d->mi.ramFree, 0)));
-	d->ramMaxFree->setText(QString("%1 (%2)").arg(KIO::convertSize(d->mi.ramMaxFree)).arg(KGlobal::locale()->formatNumber(d->mi.ramMaxFree, 0)));
-	d->ramDiskSz->setText(QString("%1 (%2)").arg(KIO::convertSize(d->mi.ramDiskSize)).arg(KGlobal::locale()->formatNumber(d->mi.ramDiskSize, 0)));
+	d->ramSize->setText(
+	    QString("%1 (%2)").arg(
+		KIO::convertSize(d->mi.ramSize)).arg(
+		    KGlobal::locale()->formatNumber(d->mi.ramSize, 0)));
+	d->ramFree->setText(
+	    QString("%1 (%2)").arg(KIO::convertSize(d->mi.ramFree)).arg(
+		KGlobal::locale()->formatNumber(d->mi.ramFree, 0)));
+	d->ramMaxFree->setText(
+	    QString("%1 (%2)").arg(KIO::convertSize(d->mi.ramMaxFree)).arg(
+		KGlobal::locale()->formatNumber(d->mi.ramMaxFree, 0)));
+	d->ramDiskSz->setText(
+	    QString("%1 (%2)").arg(KIO::convertSize(d->mi.ramDiskSize)).arg(
+		KGlobal::locale()->formatNumber(d->mi.ramDiskSize, 0)));
 
 
 	PsiTime pt(&d->mi.time, &d->mi.tz);
@@ -935,7 +977,8 @@ void PlpMachinePage::slotJobData(KIO::Job *job, const QByteArray &data) {
 	dt.setTime_t(pt.getTime());
 	d->machTime->setText(KGlobal::locale()->formatDateTime(dt, false));
 	d->machUTCo->setText(i18n("%1 seconds").arg(d->mi.tz.utc_offset));
-	d->machDST->setText((d->mi.tz.dst_zones & PsiTime::PSI_TZ_HOME) ? i18n("yes") : i18n("no"));
+	d->machDST->setText((d->mi.tz.dst_zones & PsiTime::PSI_TZ_HOME)
+			    ? i18n("yes") : i18n("no"));
 
 	ostrstream mbs;
 	mbs << d->mi.mainBatteryUsedTime << '\0';
@@ -945,18 +988,33 @@ void PlpMachinePage::slotJobData(KIO::Job *job, const QByteArray &data) {
 	d->mbattChanged->setText(KGlobal::locale()->formatDateTime(dt, false));
 	d->mbattStatus->setText(
 	    KGlobal::locale()->translate(d->mi.mainBatteryStatus));
-	d->mbattPower->setText(QString("%1 mAs").arg(KGlobal::locale()->formatNumber(d->mi.mainBatteryUsedPower, 0)));
-	d->mbattCurrent->setText(QString("%1 mA").arg(KGlobal::locale()->formatNumber(d->mi.mainBatteryCurrent, 0)));
-	d->mbattVoltage->setText(QString("%1 mV").arg(KGlobal::locale()->formatNumber(d->mi.mainBatteryVoltage, 0)));
-	d->mbattMaxVoltage->setText(QString("%1 mV").arg(KGlobal::locale()->formatNumber(d->mi.mainBatteryMaxVoltage, 0)));
+	d->mbattPower->setText(
+	    QString("%1 mAs").arg(
+		KGlobal::locale()->formatNumber(d->mi.mainBatteryUsedPower, 0)));
+	d->mbattCurrent->setText(
+	    QString("%1 mA").arg(
+		KGlobal::locale()->formatNumber(d->mi.mainBatteryCurrent, 0)));
+	d->mbattVoltage->setText(
+	    QString("%1 mV").arg(
+		KGlobal::locale()->formatNumber(d->mi.mainBatteryVoltage, 0)));
+	d->mbattMaxVoltage->setText(
+	    QString("%1 mV").arg(KGlobal::locale()->formatNumber(
+				     d->mi.mainBatteryMaxVoltage, 0)));
 
-	ostrstream bbs;
-	bbs << d->mi.backupBatteryUsedTime << '\0';
-	d->bbattUsage->setText(QString(bbs.str()));
 	d->bbattStatus->setText(
 	    KGlobal::locale()->translate(d->mi.backupBatteryStatus));
-	d->bbattVoltage->setText(QString("%1 mV").arg(KGlobal::locale()->formatNumber(d->mi.backupBatteryVoltage, 0)));
-	d->bbattMaxVoltage->setText(QString("%1 mV").arg(KGlobal::locale()->formatNumber(d->mi.backupBatteryMaxVoltage, 0)));
+	d->bbattVoltage->setText(
+	    QString("%1 mV").arg(
+		KGlobal::locale()->formatNumber(d->mi.backupBatteryVoltage, 0)));
+	d->bbattMaxVoltage->setText(
+	    QString("%1 mV").arg(KGlobal::locale()->formatNumber(
+				     d->mi.backupBatteryMaxVoltage, 0)));
+
+	ostrstream bbs;
+	bbs << d->mi.externalPowerUsedTime << '\0';
+	d->epowerUsage->setText(QString(bbs.str()));
+	d->epowerSupplied->setText(
+	    d->mi.externalPower ? i18n("Yes") : i18n("No"));
     }
 }
 
@@ -965,7 +1023,6 @@ void PlpMachinePage::slotJobFinished(KIO::Job *job) {
 
     if (mJob->error())
 	job->showErrorDialog(d->props->dialog());
-    cout << "MachInfo job has finished" << endl;
 }
 
 class PlpOwnerPage::PlpOwnerPagePrivate
@@ -991,8 +1048,9 @@ PlpOwnerPage::PlpOwnerPage( KPropertiesDialog *_props ) {
 		    i18n("This shows the owner's information of the connected device."));
 
     KIO_ARGS << int(PLP_CMD_OWNERINFO);
-    KIO::StatJob *job = new KIO::StatJob(KURL("psion:/"),
-					 KIO::CMD_SPECIAL, packedArgs, false);
+    KURL u(_props->item()->url());
+    u.setPath("/");
+    KIO::StatJob *job = new KIO::StatJob(u, KIO::CMD_SPECIAL, packedArgs, false);
     connect(job, SIGNAL(result(KIO::Job *)),
 	    SLOT(slotSpecialFinished(KIO::Job *)));
 
