@@ -1,5 +1,5 @@
 /*-*-c++-*-
- * $Id: log.cc,v 1.4 2001/03/01 00:22:22 felfert Exp $
+ * $Id: log.cc,v 1.6 2003/02/03 00:56:52 felfert Exp $
  *
  * This file is part of plptools.
  *
@@ -22,18 +22,24 @@
  *
  */
 #include "log.h"
+#include <unistd.h>
 
-logbuf::logbuf(int _level) {
+logbuf::logbuf(int level, int fd) {
     ptr = buf;
     len = 0;
-    level = _level;
+    _on = true;
+    _level = level;
+    _fd = fd;
 }
 
 int logbuf::overflow(int c) {
     if (c == '\n') {
 	*ptr++ = '\n';
 	*ptr = '\0';
-	syslog(level, buf);
+	if (_on)
+	    syslog(_level, "%s", buf);
+	else if (_fd != -1)
+	    write(_fd, buf, len + 1);
 	ptr = buf;
 	len = 0;
 	return 0;
