@@ -1,10 +1,10 @@
 /*-*-c++-*-
- * $Id: rfsv.h,v 1.18 2001/05/22 21:19:20 felfert Exp $
+ * $Id: rfsv.h,v 1.24 2002/07/14 06:35:33 felfert Exp $
  *
  * This file is part of plptools.
  *
  *  Copyright (C) 1999  Philip Proudman <philip.proudman@btinternet.com>
- *  Copyright (C) 1999-2001 Fritz Elfert <felfert@to.com>
+ *  Copyright (C) 1999-2002 Fritz Elfert <felfert@to.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -30,7 +30,7 @@
 #include <plpdirent.h>
 #include <bufferstore.h>
 
-typedef deque<class PlpDirent> PlpDir;
+typedef std::deque<class PlpDirent> PlpDir;
 
 class ppsocket;
 class PlpDrive;
@@ -53,8 +53,8 @@ class rfsv32;
  * @internal
  */
 class rfsvDirhandle {
-    friend rfsv16;
-    friend rfsv32;
+    friend class rfsv16;
+    friend class rfsv32;
 
 private:
     u_int32_t h;
@@ -247,7 +247,7 @@ public:
     *
     * @returns A Psion error code (One of enum @ref #errs ).
     */
-    virtual Enum<errs> mktemp(u_int32_t &handle, string &name) = 0;
+    virtual Enum<errs> mktemp(u_int32_t &handle, std::string &name) = 0;
 
     /**
     * Creates a named file.
@@ -405,12 +405,18 @@ public:
     * @param to Name of the destination file on the local machine.
     * @param func Pointer to a function which gets called on every read.
     * 	This function can be used to show some progress etc. May be set
-    * 	to NULL, where no callback is performed.
+    * 	to NULL, where no callback is performed. If the callback function
+    * 	returns 0, the operation is aborted and E_PSI_FILE_CANCEL is returned.
     *
     * @returns A Psion error code (One of enum @ref #errs ).
     */
     virtual Enum<errs> copyFromPsion(const char *from, const char *to, void *, cpCallback_t func) = 0;
 
+    /**
+    * Copies a file from the Psion to the local machine.
+    */
+    virtual Enum<rfsv::errs> copyFromPsion(const char *from, int fd, cpCallback_t cb) = 0;
+
     /**
     * Copies a file from local machine to the Psion.
     *
@@ -418,7 +424,8 @@ public:
     * @param to Name of the destination file on the Psion.
     * @param func Pointer to a function which gets called on every read.
     * 	This function can be used to show some progress etc. May be set
-    * 	to NULL, where no callback is performed.
+    * 	to NULL, where no callback is performed. If the callback function
+    * 	returns 0, the operation is aborted and E_PSI_FILE_CANCEL is returned.
     *
     * @returns A Psion error code (One of enum @ref #errs ).
     */
@@ -434,7 +441,8 @@ public:
     * @param to Name of the destination file.
     * @param func Pointer to a function which gets called on every read.
     * 	This function can be used to show some progress etc. May be set
-    * 	to NULL, where no callback is performed.
+    * 	to NULL, where no callback is performed. If the callback function
+    * 	returns 0, the operation is aborted and E_PSI_FILE_CANCEL is returned.
     *
     * @returns A Psion error code (One of enum @ref #errs ).
     */
@@ -574,7 +582,7 @@ public:
     * @returns Pointer to static textual representation of file attributes.
     *
     */
-    string attr2String(const u_int32_t attr);
+    std::string attr2String(const u_int32_t attr);
 
     /**
     * Converts an open-mode (A combination of the PSI_O_ constants.)
@@ -590,7 +598,21 @@ public:
     /**
     * Utility method, converts '/' to '\'.
     */
-    static string convertSlash(const string &name);
+    static std::string convertSlash(const std::string &name);
+
+    /**
+     * Retrieve speed of serial link.
+     *
+     * @returns The speed of the serial link in baud or -1 on error.
+     */
+    int getSpeed();
+
+    /**
+     * Retrieves the protocol version.
+     *
+     * @returns Either 3 or 5 representing Series 3 (SIBO) or Series 5 (EPOC)
+     */
+    virtual int getProtocolVersion() = 0;
 
 protected:
     /**
