@@ -1,5 +1,5 @@
 /*-*-c++-*-
- * $Id: rfsv32.cc,v 1.21 2001/05/22 21:19:20 felfert Exp $
+ * $Id: rfsv32.cc,v 1.24 2002/07/14 06:35:33 felfert Exp $
  *
  * This file is part of plptools.
  *
@@ -38,6 +38,8 @@
 #include "bufferarray.h"
 #include "plpdirent.h"
 
+using namespace std;
+
 rfsv32::rfsv32(ppsocket * _skt)
 {
     skt = _skt;
@@ -428,12 +430,12 @@ Enum<rfsv::errs> rfsv32::
 fread(const u_int32_t handle, unsigned char * const buf, const u_int32_t len, u_int32_t &count)
 {
     Enum<rfsv::errs> res;
+    bufferStore a;
     count = 0;
     long l;
     unsigned char *p = buf;
 
     do {
-	bufferStore a;
 	a.addDWord(handle);
 	a.addDWord(((len - count) > RFSV_SENDLEN)?RFSV_SENDLEN:(len - count));
 	if (!sendCommand(READ_FILE, a))
@@ -445,6 +447,7 @@ fread(const u_int32_t handle, unsigned char * const buf, const u_int32_t len, u_
 	    count += l;
 	    p += l;
 	}
+	a.init();
     } while ((count < len) && (l > 0));
     return res;
 }
@@ -493,18 +496,42 @@ copyFromPsion(const char *from, const char *to, void *ptr, cpCallback_t cb)
     unsigned char *buff = new unsigned char[RFSV_SENDLEN];
     do {
 	if ((res = fread(handle, buff, RFSV_SENDLEN, len)) == E_PSI_GEN_NONE) {
-	    op.write(buff, len);
+	    op.write((char *)buff, len);
 	    total += len;
 	    if (cb && !cb(ptr, total))
 		res = E_PSI_FILE_CANCEL;
 	}
     } while ((len > 0) && (res == E_PSI_GEN_NONE));
-    delete[]buff;
+    delete [] buff;
     fclose(handle);
     op.close();
     return res;
 }
 
+Enum<rfsv::errs> rfsv32::
+copyFromPsion(const char *from, int fd, cpCallback_t cb)
+{
+    Enum<rfsv::errs> res;
+    u_int32_t handle;
+    u_int32_t len;
+    u_int32_t total = 0;
+
+    if ((res = fopen(EPOC_OMODE_SHARE_READERS | EPOC_OMODE_BINARY, from, handle)) != E_PSI_GEN_NONE)
+	return res;
+    unsigned char *buff = new unsigned char[RFSV_SENDLEN];
+    do {
+	if ((res = fread(handle, buff, RFSV_SENDLEN, len)) == E_PSI_GEN_NONE) {
+	    write(fd, buff, len);
+	    total += len;
+	    if (cb && !cb(NULL, total))
+		res = E_PSI_FILE_CANCEL;
+	}
+    } while ((len > 0) && (res == E_PSI_GEN_NONE));
+    delete [] buff;
+    fclose(handle);
+    return res;
+}
+
 Enum<rfsv::errs> rfsv32::
 copyToPsion(const char *from, const char *to, void *ptr, cpCallback_t cb)
 {
@@ -524,7 +551,7 @@ copyToPsion(const char *from, const char *to, void *ptr, cpCallback_t cb)
     u_int32_t total = 0;
     while (ip && !ip.eof() && (res == E_PSI_GEN_NONE)) {
 	u_int32_t len;
-	ip.read(buff, RFSV_SENDLEN);
+	ip.read((char *)buff, RFSV_SENDLEN);
 	if ((res = fwrite(handle, buff, ip.gcount(), len)) == E_PSI_GEN_NONE) {
 	    total += len;
 	    if (cb && !cb(ptr, total))
