@@ -1,5 +1,5 @@
 /*-*-c++-*-
- * $Id: ncp.cc,v 1.15 2001/08/01 04:20:30 felfert Exp $
+ * $Id: ncp.cc,v 1.22 2002/07/16 19:40:29 felfert Exp $
  *
  * This file is part of plptools.
  *
@@ -29,40 +29,51 @@
 #include <string>
 #include <time.h>
 
+#include <bufferstore.h>
+#include <bufferarray.h>
+#include <rfsv.h>
+
 #include "ncp.h"
 #include "linkchan.h"
-#include <bufferstore.h>
 #include "link.h"
-#include <bufferarray.h>
+#include "main.h"
 
 #define MAX_CHANNELS_PSION 256
 #define MAX_CHANNELS_SIBO  8
 #define NCP_SENDLEN 250
 
-ncp::ncp(const char *fname, int baud, IOWatch & iow)
+using namespace std;
+
+ncp::ncp(const char *fname, int baud, unsigned short _verbose)
 {
     channelPtr = new channel*[MAX_CHANNELS_PSION + 1];
+    assert(channelPtr);
     messageList = new bufferStore[MAX_CHANNELS_PSION + 1];
+    assert(messageList);
     remoteChanList = new int[MAX_CHANNELS_PSION + 1];
+    assert(remoteChanList);
 
-    l = new link(fname, baud, iow);
     failed = false;
-    verbose = 0;
+    verbose = _verbose;
 
     // until detected on receipt of INFO we use these.
     maxChannels = MAX_CHANNELS_SIBO;
     protocolVersion = PV_SERIES_5;
+    lChan = NULL;
 
     // init channels
     for (int i = 0; i < MAX_CHANNELS_PSION; i++)
 	channelPtr[i] = NULL;
+
+    l = new Link(fname, baud, this, verbose);
+    assert(l);
 }
 
 ncp::~ncp()
 {
     bufferStore b;
     for (int i = 0; i < maxLinks(); i++) {
-	if (channelPtr[i]) {
+	if (isValidChannel(i)) {
 	    bufferStore b2;
 	    b2.addByte(remoteChanList[i]);
 	    controlChannel(i, NCON_MSG_CHANNEL_DISCONNECT, b2);
@@ -71,8 +82,9 @@ ncp::~ncp()
     }
     controlChannel(0, NCON_MSG_NCP_END, b);
     delete l;
-    delete channelPtr;
-    delete remoteChanList;
+    delete [] channelPtr;
+    delete [] remoteChanList;
+    delete [] messageList;
 }
 
 int ncp::
@@ -83,50 +95,29 @@ maxLinks() {
 void ncp::
 reset() {
     for (int i = 0; i < maxLinks(); i++) {
-	if (channelPtr[i])
+	if (isValidChannel(i))
 	    channelPtr[i]->terminateWhenAsked();
 	channelPtr[i] = NULL;
     }
     failed = false;
+    if (lChan)
+	delete(lChan);
     lChan = NULL;
     protocolVersion = PV_SERIES_5; // until detected on receipt of INFO
     l->reset();
 }
 
-short int ncp::
+unsigned short ncp::
 getVerbose()
 {
     return verbose;
 }
 
 void ncp::
-setVerbose(short int _verbose)
+setVerbose(unsigned short _verbose)
 {
     verbose = _verbose;
-}
-
-short int ncp::
-getLinkVerbose()
-{
-    return l->getVerbose();
-}
-
-void ncp::
-setLinkVerbose(short int _verbose)
-{
-    l->setVerbose(_verbose);
-}
-
-short int ncp::
-getPktVerbose()
-{
-    return l->getPktVerbose();
-}
-
-void ncp::
-setPktVerbose(short int _verbose)
-{
-    l->setPktVerbose(_verbose);
+    l->setVerbose(verbose);
 }
 
 short int ncp::
@@ -136,34 +127,29 @@ getProtocolVersion()
 }
 
 void ncp::
-poll()
-{
-    bufferArray res(l->poll());
-    while (!res.empty()) {
-	bufferStore s = res.pop();
-	if (s.getLen() > 1) {
-	    int channel = s.getByte(0);
-	    s.discardFirstBytes(1);
-	    if (channel == 0) {
-		decodeControlMessage(s);
+receive(bufferStore s) {
+    if (s.getLen() > 1) {
+	int channel = s.getByte(0);
+	s.discardFirstBytes(1);
+	if (channel == 0) {
+	    decodeControlMessage(s);
+	} else {
+	    int allData = s.getByte(1);
+	    s.discardFirstBytes(2);
+	    if (!isValidChannel(channel)) {
+		lerr << "ncp: Got message for unknown channel\n";
 	    } else {
-		int allData = s.getByte(1);
-		s.discardFirstBytes(2);
-		if (channelPtr[channel] == NULL) {
-		    cerr << "ncp: Got message for unknown channel\n";
-		} else {
-		    messageList[channel].addBuff(s);
-		    if (allData == LAST_MESS) {
-			channelPtr[channel]->ncpDataCallback(messageList[channel]);
-			messageList[channel].init();
-		    } else if (allData != NOT_LAST_MESS) {
-			cerr << "ncp: bizarre third byte!\n";
-		    }
+		messageList[channel].addBuff(s);
+		if (allData == LAST_MESS) {
+		    channelPtr[channel]->ncpDataCallback(messageList[channel]);
+		    messageList[channel].init();
+		} else if (allData != NOT_LAST_MESS) {
+		    lerr << "ncp: bizarre third byte!\n";
 		}
 	    }
-	} else
-	    cerr << "Got null message\n";
-    }
+	}
+    } else
+	lerr << "Got null message\n";
 }
 
 void ncp::
@@ -176,10 +162,39 @@ controlChannel(int chan, enum interControllerMessageType t, bufferStore & comman
     open.addByte(t);
     open.addBuff(command);
     if (verbose & NCP_DEBUG_LOG)
-	cout << "ncp: >> " << ctrlMsgName(t) << " " << chan << endl;
+	lout << "ncp: >> " << ctrlMsgName(t) << " " << chan << endl;
     l->send(open);
 }
 
+PcServer *ncp::
+findPcServer(const char *name)
+{
+    if (name) {
+	vector<PcServer>::iterator i;
+	for (i = pcServers.begin(); i != pcServers.end(); i++)
+	    if (i->getName() == name)
+		    return i->self();
+    }
+    return NULL;
+}
+
+void ncp::
+registerPcServer(ppsocket *skt, const char *name) {
+    pcServers.push_back(PcServer(skt, name));
+}
+
+void ncp::
+unregisterPcServer(PcServer *server) {
+    if (server) {
+	vector<PcServer>::iterator i;
+	for (i = pcServers.begin(); i != pcServers.end(); i++)
+	    if (i->self() == server) {
+		pcServers.erase(i);
+		return;
+	    }
+    }
+}
+
 void ncp::
 decodeControlMessage(bufferStore & buff)
 {
@@ -188,46 +203,77 @@ decodeControlMessage(bufferStore & buff)
     interControllerMessageType imt = (interControllerMessageType)buff.getByte(1);
     buff.discardFirstBytes(2);
     if (verbose & NCP_DEBUG_LOG)
-	cout << "ncp: << " << ctrlMsgName(imt) << " " << remoteChan;
+	lout << "ncp: << " << ctrlMsgName(imt) << " " << remoteChan;
 
     bufferStore b;
+    int localChan;
 
     switch (imt) {
 	case NCON_MSG_CONNECT_TO_SERVER:
 	    if (verbose & NCP_DEBUG_LOG) {
 		if (verbose & NCP_DEBUG_DUMP)
-		    cout << " [" << buff << "]";
-		cout << endl;
+		    lout << " [" << buff << "]";
+		lout << endl;
 	    }
 
-	    int localChan;
-
-	    // Ack with connect response
-	    localChan = getFirstUnusedChan();
-	    b.addByte(remoteChan);
-	    b.addByte(0x0);
-	    controlChannel(localChan, NCON_MSG_CONNECT_RESPONSE, b);
-
-	    // NOTE: we don't allow connections from the
-	    // Psion to any local "processes" other than
-	    // LINK.* - Matt might need to change this for
-	    // his NCP<->TCP/IP bridge code...
-
+	    failed = false;
 	    if (!strcmp(buff.getString(0), "LINK.*")) {
 		if (lChan)
-		    failed = true;
-		if (verbose & NCP_DEBUG_LOG)
-		    cout << "ncp: Link UP" << endl;
-		channelPtr[localChan] = lChan = new linkChan(this);
-		lChan->setNcpChannel(localChan);
-		lChan->ncpConnectAck();
-		lChan->setVerbose(verbose);
-	    } else {
-		if (verbose & NCP_DEBUG_LOG)
-		    cout << "ncp: REJECT connect" << endl;
-		bufferStore b;
+		    localChan = lChan->getNcpChannel();
+		else
+		    localChan = getFirstUnusedChan();
+
+		// Ack with connect response
 		b.addByte(remoteChan);
-		controlChannel(0, NCON_MSG_CHANNEL_DISCONNECT, b);
+		b.addByte(0);
+		controlChannel(localChan, NCON_MSG_CONNECT_RESPONSE, b);
+		if (verbose & NCP_DEBUG_LOG)
+		    lout << "ncp: Link UP" << endl;
+		linf << _("Connected with a S")
+		     << ((protocolVersion == PV_SERIES_5) ? 5 : 3) << _(" at ")
+		     << getSpeed() << _("baud") << endl;
+		// Create linkchan if it does not yet exist
+		if (!lChan) {
+		    if (verbose & NCP_DEBUG_LOG)
+			lout << "ncp: new passive linkChan" << endl;
+		    channelPtr[localChan] =
+			lChan = new linkChan(this, localChan);
+		    lChan->setVerbose(verbose);
+		}
+		lChan->ncpConnectAck();
+	    } else {
+		PcServer *s = findPcServer(buff.getString(0));
+		bool ok = false;
+
+		if (s) {
+		    localChan = getFirstUnusedChan();
+		    ok = s->clientConnect(localChan, remoteChan);
+		    if (!ok)
+			// release channel ptr
+			channelPtr[localChan] = NULL;
+		}
+		b.addByte(remoteChan);
+		if (ok) {
+		    b.addByte(rfsv::E_PSI_GEN_NONE);
+		    if (verbose & NCP_DEBUG_LOG)
+			lout << "ncp: ACCEPT client connect" << endl;
+		} else {
+		    localChan = 0;
+		    b.addByte(rfsv::E_PSI_FILE_NXIST);
+		    if (verbose & NCP_DEBUG_LOG)
+			lout << "ncp: REJECT client connect" << endl;
+		}
+		controlChannel(localChan, NCON_MSG_CONNECT_RESPONSE, b);
+
+		// Create linkchan if it does not yet exist
+		if (!lChan) {
+		    if (verbose & NCP_DEBUG_LOG)
+			lout << "ncp: new active linkChan" << endl;
+		    channelPtr[localChan] =
+			lChan = new linkChan(this, -1);
+		    lChan->setVerbose(verbose);
+		}
+
 	    }
 	    break;
 
@@ -235,23 +281,25 @@ decodeControlMessage(bufferStore & buff)
 
 	    int forChan;
 
+	    failed = false;
 	    forChan = buff.getByte(0);
 	    if (verbose & NCP_DEBUG_LOG)
-		cout << " ch=" << forChan << " stat=";
+		lout << " ch=" << forChan << " stat=";
 	    if (buff.getByte(1) == 0) {
 		if (verbose & NCP_DEBUG_LOG)
-		    cout << "OK" << endl;
-		if (channelPtr[forChan]) {
+		    lout << "OK" << endl;
+		if (isValidChannel(forChan)) {
 		    remoteChanList[forChan] = remoteChan;
 		    channelPtr[forChan]->ncpConnectAck();
 		} else {
 		    if (verbose & NCP_DEBUG_LOG)
-			cout << "ncp: message for unknown channel" << endl;
+			lout << "ncp: message for unknown channel" << endl;
 		}
 	    } else {
 		if (verbose & NCP_DEBUG_LOG)
-		    cout << "Unknown " << (int) buff.getByte(1) << endl;
-		channelPtr[forChan]->ncpConnectNak();
+		    lout << "Unknown " << (int) buff.getByte(1) << endl;
+		if (isValidChannel(forChan))
+		    channelPtr[forChan]->ncpConnectNak();
 	    }
 	    break;
 
@@ -259,6 +307,7 @@ decodeControlMessage(bufferStore & buff)
 
 	    int ver;
 
+	    failed = false;
 	    ver = buff.getByte(0);
 	    // Series 3c returns '3', as does mclink. PsiWin 1.1
 	    // returns version 2. We return whatever version we're
@@ -271,8 +320,8 @@ decodeControlMessage(bufferStore & buff)
 		protocolVersion = ver;
 		if (verbose & NCP_DEBUG_LOG) {
 		    if (verbose & NCP_DEBUG_DUMP)
-			cout << " [" << buff << "]";
-		    cout << endl;
+			lout << " [" << buff << "]";
+		    lout << endl;
 		}
 		// Fake NCP version 2 for a Series 3 (behave like PsiWin 1.1)
 		if (ver == PV_SERIES_3) {
@@ -285,16 +334,17 @@ decodeControlMessage(bufferStore & buff)
 		// Do we send a time of 0 or a real time?
 		// The Psion uses this to determine whether to
 		// restart. (See protocol docs for details)
-		// b.addDWord(0);
 		b.addDWord(time(NULL));
 		controlChannel(0, NCON_MSG_NCP_INFO, b);
-	    } else
-		cout << "ALERT!!!! Unexpected Protocol Version!! (No Series 5/3?)!" << endl;
+	    } else {
+		lout << "ALERT!!!! Unexpected Protocol Version!! (No Series 5/3?)!" << endl;
+		failed = true;
+	    }
 	    break;
 
 	case NCON_MSG_CHANNEL_DISCONNECT:
 	    if (verbose & NCP_DEBUG_LOG)
-		cout << " ch=" << (int) buff.getByte(0) << endl;
+		lout << " ch=" << (int) buff.getByte(0) << endl;
 	    disconnect(buff.getByte(0));
 	    l->purgeQueue(remoteChan);
 	    break;
@@ -306,8 +356,8 @@ decodeControlMessage(bufferStore & buff)
 	default:
 	    if (verbose & NCP_DEBUG_LOG) {
 		if (verbose & NCP_DEBUG_DUMP)
-		    cout << " [" << buff << "]";
-		cout << endl;
+		    lout << " [" << buff << "]";
+		lout << endl;
 	    }
 
     }
@@ -319,27 +369,34 @@ getFirstUnusedChan()
     for (int cNum = 1; cNum < maxLinks(); cNum++) {
 	if (channelPtr[cNum] == NULL) {
 	    if (verbose & NCP_DEBUG_LOG)
-		cout << "ncp: getFirstUnusedChan=" << cNum << endl;
+		lout << "ncp: getFirstUnusedChan=" << cNum << endl;
+	    channelPtr[cNum] = (channel *)0xdeadbeef;
 	    return cNum;
 	}
     }
     return 0;
 }
 
+bool ncp::
+isValidChannel(int channel)
+{
+    return (channelPtr[channel] && ((int)channelPtr[channel] != 0xdeadbeef));
+}
+
 void ncp::
 RegisterAck(int chan, const char *name)
 {
     if (verbose & NCP_DEBUG_LOG)
-	cout << "ncp: RegisterAck: chan=" << chan << endl;
+	lout << "ncp: RegisterAck: chan=" << chan << endl;
     for (int cNum = 1; cNum < maxLinks(); cNum++) {
 	channel *ch = channelPtr[cNum];
-	if (ch && ch->getNcpChannel() == chan) {
+	if (isValidChannel(cNum) && ch->getNcpChannel() == chan) {
 	    ch->setNcpConnectName(name);
 	    ch->ncpRegisterAck();
 	    return;
 	}
     }
-    cerr << "ncp: RegisterAck: no channel to deliver" << endl;
+    lerr << "ncp: RegisterAck: no channel to deliver" << endl;
 }
 
 void ncp::
@@ -354,9 +411,9 @@ Register(channel * ch)
 	    ch->setNcpChannel(cNum);
 	    lChan->Register(ch);
 	} else
-	    cerr << "ncp: Out of channels in register" << endl;
+	    lerr << "ncp: Out of channels in register" << endl;
     } else
-	cerr << "ncp: Register without established lChan" << endl;
+	lerr << "ncp: Register without established lChan" << endl;
 }
 
 int ncp::
@@ -413,13 +470,13 @@ send(int channel, bufferStore & a)
 void ncp::
 disconnect(int channel)
 {
-    if (channelPtr[channel] == NULL) {
-	cerr << "ncp: Ignored disconnect for unknown channel #" << channel << endl;
+    if (!isValidChannel(channel)) {
+	lerr << "ncp: Ignored disconnect for unknown channel #" << channel << endl;
 	return;
     }
     channelPtr[channel]->terminateWhenAsked();
     if (verbose & NCP_DEBUG_LOG)
-	cout << "ncp: disconnect: channel=" << channel << endl;
+	lout << "ncp: disconnect: channel=" << channel << endl;
     channelPtr[channel] = NULL;
     bufferStore b;
     b.addByte(remoteChanList[channel]);
@@ -435,9 +492,20 @@ stuffToSend()
 bool ncp::
 hasFailed()
 {
-    if (failed)
-	return true;
-    return l->hasFailed();
+    bool lfailed = l->hasFailed();
+    if (failed || lfailed) {
+	if (verbose & NCP_DEBUG_LOG)
+	    lout << "ncp: hasFailed: " << failed << ", " << lfailed << endl;
+    }
+    failed |= lfailed;
+    if (failed) {
+	if (lChan) {
+	    channelPtr[lChan->getNcpChannel()] = NULL;
+	    delete lChan;
+	}
+	lChan = NULL;
+    }
+    return failed;
 }
 
 bool ncp::
@@ -446,6 +514,12 @@ gotLinkChannel()
     return (lChan != NULL);
 }
 
+int ncp::
+getSpeed()
+{
+    return l->getSpeed();
+}
+
 char *ncp::
 ctrlMsgName(unsigned char msgType)
 {
