@@ -1,10 +1,10 @@
 /*-*-c++-*-
- * $Id: ftp.cc,v 1.27 2001/07/26 13:07:36 felfert Exp $
+ * $Id: ftp.cc,v 1.38 2002/07/14 06:35:33 felfert Exp $
  *
  * This file is part of plptools.
  *
  *  Copyright (C) 1999  Philip Proudman <philip.proudman@btinternet.com>
- *  Copyright (C) 1999-2001 Fritz Elfert <felfert@to.com>
+ *  Copyright (C) 1999-2002 Fritz Elfert <felfert@to.com>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  */
-#define EXPERIMENTAL
+#undef EXPERIMENTAL
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -59,6 +59,8 @@ extern "C"  {
 }
 #endif
 
+using namespace std;
+
 static char psionDir[1024];
 static rfsv *comp_a;
 static int continueRunning;
@@ -115,6 +117,7 @@ void ftp::usage() {
     cout << "  runrestore <unixfile>" << endl;
     cout << "  machinfo" << endl;
     cout << "  ownerinfo" << endl;
+    cout << "  setupinfo" << endl;
 }
 
 static int Wildmat(const char *s, char *p);
@@ -208,15 +211,14 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
     Enum<rfsv::errs> res;
     bool prompt = true;
     bool hash = false;
-    bool S5mx = false;
     cpCallback_t cab = checkAbortNoHash;
     bool once = false;
 
-    if (xargc > 1) {
+    if (xargc) {
 	once = true;
-	argc = (xargc<11)?xargc-1:10;
+	argc = (xargc<10)?xargc:10;
 	for (int i = 0; i < argc; i++)
-	    argv[i] = xargv[i+1];
+	    argv[i] = xargv[i];
     }
     {
 	Enum<rpcs::machs> machType;
@@ -224,18 +226,13 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	if ((res = r.getOwnerInfo(b)) == rfsv::E_PSI_GEN_NONE) {
 	    r.getMachineType(machType);
 	    if (!once) {
-		cout << _("Connected to a ") << machType << _(", OwnerInfo:") << endl;
+		int speed = a.getSpeed();
+		cout << _("Connected to a ") << machType << _(" at ")
+		     << speed << _(" baud, OwnerInfo:") << endl;
 		while (!b.empty())
 		    cout << "  " << b.pop().getString() << endl;
 		cout << endl;
 	    }
-	    if (machType == rpcs::PSI_MACH_S5) {
-		rpcs::machineInfo mi;
-		if ((res = r.getMachineInfo(mi)) == rfsv::E_PSI_GEN_NONE) {
-		    if (!strcmp(mi.machineName, "SERIES5mx"))
-			S5mx = true;
-		}
-	    }
 	} else
 	    cerr << _("OwnerInfo returned error ") << res << endl;
     }
@@ -257,7 +254,7 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	    }
 	}
 	if (!strcmp(defDrive, "::")) {
-	    cerr << _("FATAL: Couln't find default Drive") << endl;
+	    cerr << _("FATAL: Couldn't find default Drive") << endl;
 	    return -1;
 	}
     } else
@@ -275,7 +272,7 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	if (!once)
 	    getCommand(argc, argv);
 
-	if (!strcmp(argv[0], "help")) {
+	if ((!strcmp(argv[0], "help")) || (!strcmp(argv[0], "?"))) {
 	    usage();
 	    continue;
 	}
@@ -356,7 +353,11 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	    if ((res = a.fgetmtime(f1, mtime)) != rfsv::E_PSI_GEN_NONE)
 		cerr << _("Error: ") << res << endl;
 	    else
-		cout << mtime << endl;
+		cout << mtime << "(" << hex
+		     << setw(8) << setfill('0') << mtime.getPsiTimeHi()
+		     << ":"
+		     << setw(8) << setfill('0') << mtime.getPsiTimeLo()
+		     << ")" << endl;
 	    continue;
 	}
 	if (!strcmp(argv[0], "sattr") && (argc == 3)) {
@@ -420,12 +421,16 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 
 		    if ((devbits & 1) != 0) {
 			if (a.devinfo(i + 'A', drive) == rfsv::E_PSI_GEN_NONE)
-			    cout << (char) ('A' + i) << "     " <<
-				hex << setw(4) << setfill('0') << drive.getMediaType() << " " <<
-				setw(12) << setfill(' ') << setiosflags(ios::left) <<
-				drive.getName() << resetiosflags(ios::left) << dec << setw(9) <<
-				drive.getSize() << setw(9) << drive.getSpace() << "  " << setw(8) << setfill('0') << hex <<
-				drive.getUID() << endl;
+			    cout << (char) ('A' + i) << "     " << hex
+				 << setw(4) << setfill('0')
+				 << drive.getMediaType() << " " << setw(12)
+				 << setfill(' ') << setiosflags(ios::left)
+				 << drive.getName().c_str()
+				 << resetiosflags(ios::left) << dec << setw(9)
+				 << drive.getSize() << setw(9)
+				 << drive.getSpace() << "  " << setw(8)
+				 << setfill('0') << hex << drive.getUID()
+				 << dec << endl;
 		    }
 		    devbits >>= 1;
 		}
@@ -435,7 +440,43 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	}
 	if (!strcmp(argv[0], "ls") || !strcmp(argv[0], "dir")) {
 	    PlpDir files;
-	    if ((res = a.dir(psionDir, files)) != rfsv::E_PSI_GEN_NONE)
+	    char dtmp[1024];
+	    char *dname = psionDir;
+
+	    if (argc > 1) {
+		u_int32_t tmp;
+		strcpy(dtmp, psionDir);
+		if (!strcmp(argv[1], "..")) {
+		    strcpy(f1, psionDir);
+		    char *p = f1 + strlen(f1);
+		    if (p > f1)
+			p--;
+		    *p = '\0';
+		    while ((p > f1) && (*p != '/') && (*p != '\\'))
+			p--;
+		    *(++p) = '\0';
+		    if (strlen(f1) < 3) {
+			strcpy(f1, psionDir);
+			f1[3] = '\0';
+		    }
+		} else {
+		    if ((argv[1][0] != '/') && (argv[1][0] != '\\') &&
+			(argv[1][1] != ':')) {
+			strcpy(f1, psionDir);
+			strcat(f1, argv[1]);
+		    } else
+			strcpy(f1, argv[1]);
+		}
+		if ((f1[strlen(f1) -1] != '/') && (f1[strlen(f1) -1] != '\\'))
+		    strcat(f1,"\\");
+		for (char *p = f1; *p; p++)
+		    if (*p == '/')
+			*p = '\\';
+		strcpy(dtmp, f1);
+		dname = dtmp;
+	    }
+
+	    if ((res = a.dir(dname, files)) != rfsv::E_PSI_GEN_NONE)
 		cerr << _("Error: ") << res << endl;
 	    else
 		while (!files.empty()) {
@@ -533,7 +574,7 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 		dt += dsec;
 		stat(f2, &stbuf);
 		float cps = (float)(stbuf.st_size) / dt;
-		cout << _("Transfer complete, (") << stbuf.st_size
+		cout << _("Transfer complete, (") << dec << stbuf.st_size
 		     << _(" bytes in ") << dsec << "."
 		     << dhse << _(" secs = ") << cps << " cps)\n";
 	    }
@@ -624,7 +665,7 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 		dt += dsec;
 		stat(f1, &stbuf);
 		float cps = (float)(stbuf.st_size) / dt;
-		cout << _("Transfer complete, (") << stbuf.st_size
+		cout << _("Transfer complete, (") << dec << stbuf.st_size
 		     << _(" bytes in ") << dsec << "."
 		     << dhse << _(" secs = ") << cps << " cps)\n";
 	    }
@@ -727,14 +768,81 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	// RPCS commands
 #ifdef EXPERIMENTAL
 	if (!strcmp(argv[0], "x")) {
-	    r.regOpenIter();
-	    continue;
-	}
-	if (!strcmp(argv[0], "y")) {
-	    r.configRead();
+	    u_int16_t hhh;
+	    if (r.regOpenIter((u_int32_t)-1, "%PDF-", hhh)
+		== rfsv::E_PSI_GEN_NONE) {
+		Enum<rfsv::errs> res;
+		while ((res = r.regReadIter(hhh)) == rfsv::E_PSI_GEN_NONE)
+			;;
+	    }
 	    continue;
 	}
 #endif
+	if (!strcmp(argv[0], "setupinfo")) {
+	    Enum<rfsv::errs> res;
+	    bufferStore db;
+
+	    if ((res = r.configRead(0, db)) != rfsv::E_PSI_GEN_NONE) {
+		cerr << _("Error: ") << res << endl;
+		continue;
+	    }
+	    if (db.getLen() < 1152) {
+		cerr << _("Unknown setup info received") << endl;
+		continue;
+	    }
+	    cout << _("Setup information:") << endl;
+	    cout << _("  Screen contrast:                  ") << dec
+		 << db.getDWord(0x4c) + 1 << endl;
+	    cout << _("  Keyboard click:                   ")
+		 << (db.getDWord(0x200) ?
+		(db.getDWord(0x204) ? _("high") : _("low")) : _("off")) << endl;
+	    cout << _("  Screen click:                     ")
+		 << (db.getDWord(0x20c) ?
+		(db.getDWord(0x210) ? _("high") : _("low")) : _("off")) << endl;
+	    cout << _("  Error sound:                      ")
+		 << (db.getDWord(0x214) ?
+		(db.getDWord(0x218) ? _("high") : _("low")) : _("off")) << endl;
+	    cout << _("  Auto-switch off:                  ");
+	    switch (db.getDWord(0x228)) {
+		case 0:
+		    cout << _("never");
+		    break;
+		case 1:
+		    cout << _("if running on battery power");
+		    break;
+		case 2:
+		    cout << _("always");
+		    break;
+	    }
+	    cout << endl;
+	    if (db.getDWord(0x228) != 0)
+		cout << _("  Switch off after:                 ") << dec
+		     << db.getDWord(0x22c) << _(" seconds") << endl;
+	    cout << _("  Backlight off after:              ") << dec
+		 << db.getDWord(0x234) << _(" seconds") << endl;
+	    cout << _("  Switch on when tapping on screen: ")
+		 << (db.getDWord(0x238) ? _("yes") : _("no")) << endl;
+	    cout << _("  Switch on when opening:           ")
+		 << (db.getDWord(0x23c) ? _("yes") : _("no")) << endl;
+	    cout << _("  Switch off when closing:          ")
+		 << (db.getDWord(0x23c) ? _("yes") : _("no")) << endl;
+	    cout << _("  Ask for password on startup:      ")
+		 << (db.getDWord(0x29c) ? _("yes") : _("no")) << endl;
+	    cout << _("  Show Owner info on startup:       ");
+	    switch (db.getByte(0x3b0)) {
+		case 0x31:
+		    cout << _("never");
+		    break;
+		case 0x32:
+		    cout << _("once a day");
+		    break;
+		case 0x33:
+		    cout << _("always");
+		    break;
+	    }
+	    cout << endl;
+	    continue;
+	}
 	if (!strcmp(argv[0], "run") && (argc >= 2)) {
 	    char argbuf[1024];
 	    char cmdbuf[1024];
@@ -747,7 +855,7 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 		} else
 		    strcpy(argbuf, argv[i]);
 	    }
-	    if (argv[1][1] != ':') {
+	    if (!strchr(argv[1], ':')) {
 		strcpy(cmdbuf, psionDir);
 		strcat(cmdbuf, argv[1]);
 	    } else
@@ -812,7 +920,10 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	    cout << _("  Status:       ") << mi.backupBatteryStatus << endl;
 	    cout << _("  Voltage:      ") << mi.backupBatteryVoltage << " mV" << endl;
 	    cout << _("  Max. voltage: ") << mi.backupBatteryMaxVoltage << " mV" << endl;
-	    cout << _("  Used for:     ") << mi.backupBatteryUsedTime << endl;
+	    cout << _("External power:") << endl;
+	    cout << _("  Supplied:     ")
+		 << (mi.externalPower ? _("yes") : _("no")) << endl;
+	    cout << _("  Used for:     ") << mi.externalPowerUsedTime << endl;
 	    continue;
 	}
 	if (!strcmp(argv[0], "runrestore") && (argc == 2)) {
@@ -878,8 +989,8 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	    continue;
 	}
 	if (!strcmp(argv[0], "killsave") && (argc == 2)) {
-	    bufferArray tmp;
-	    if ((res = r.queryDrive('C', tmp)) != rfsv::E_PSI_GEN_NONE)
+	    processList tmp;
+	    if ((res = r.queryPrograms(tmp)) != rfsv::E_PSI_GEN_NONE)
 		cerr << _("Error: ") << res << endl;
 	    else {
 		ofstream op(argv[1]);
@@ -888,51 +999,31 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 		    continue;
 		}
 		op << "#plpftp processlist" << endl;
-		while (!tmp.empty()) {
-		    char pbuf[128];
-		    bufferStore cmdargs;
-		    bufferStore bs = tmp.pop();
-		    int pid = bs.getWord(0);
-		    const char *proc = bs.getString(2);
-		    if (S5mx)
-			sprintf(pbuf, "%s.$%02d", proc, pid);
-		    else
-			sprintf(pbuf, "%s.$%d", proc, pid);
-		    bs = tmp.pop();
-		    if (r.getCmdLine(pbuf, cmdargs) == 0)
-			op << cmdargs.getString(0) << " " << bs.getString(0) << endl;
-		    r.stopProgram(pbuf);
+		processList::iterator i;
+		for (i = tmp.begin(); i != tmp.end(); i++) {
+		    op << i->getArgs() << endl;
+		    r.stopProgram(i->getProcId());
 		}
 		op.close();
 	    }
 	    continue;
 	}
 	if (!strcmp(argv[0], "kill") && (argc >= 2)) {
-	    bufferArray tmp, tmp2;
+	    processList tmp;
 	    bool anykilled = false;
-	    if ((res = r.queryDrive('C', tmp)) != rfsv::E_PSI_GEN_NONE)
+	    if ((res = r.queryPrograms(tmp)) != rfsv::E_PSI_GEN_NONE)
 		cerr << _("Error: ") << res << endl;
 	    else {
-		tmp2 = tmp;
 		for (int i = 1; i < argc; i++) {
 		    int kpid;
-		    tmp = tmp2;
 		    if (!strcmp(argv[i], "all"))
 			kpid = -1;
 		    else
 			sscanf(argv[i], "%d", &kpid);
-		    while (!tmp.empty()) {
-			bufferStore bs = tmp.pop();
-			tmp.pop();
-			int pid = bs.getWord(0);
-			const char *proc = bs.getString(2);
-			if (kpid == -1 || kpid == pid) {
-			    char pbuf[128];
-			    if (S5mx)
-				sprintf(pbuf, "%s.$%02d", proc, pid);
-			    else
-				sprintf(pbuf, "%s.$%d", proc, pid);
-			    r.stopProgram(pbuf);
+		    processList::iterator j;
+		    for (j = tmp.begin(); j != tmp.end(); j++) {
+			if (kpid == -1 || kpid == j->getPID()) {
+			    r.stopProgram(j->getProcId());
 			    anykilled = true;
 			}
 		    }
@@ -945,20 +1036,13 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 	    continue;
 	}
 	if (!strcmp(argv[0], "ps")) {
-	    bufferArray tmp;
-	    if ((res = r.queryDrive('C', tmp)) != rfsv::E_PSI_GEN_NONE)
+	    processList tmp;
+	    if ((res = r.queryPrograms(tmp)) != rfsv::E_PSI_GEN_NONE)
 		cerr << _("Error: ") << res << endl;
 	    else {
 		cout << "PID   CMD          ARGS" << endl;
-		while (!tmp.empty()) {
-		    bufferStore bs = tmp.pop();
-		    bufferStore bs2 = tmp.pop();
-		    int pid = bs.getWord(0);
-		    const char *proc = bs.getString(2);
-		    const char *arg = bs2.getString();
-
-		    printf("%5d %-12s %s\n", pid, proc, arg);
-		}
+		for (processList::iterator i = tmp.begin(); i != tmp.end(); i++)
+		    cout << *i << endl;
 	    }
 	    continue;
 	}
@@ -971,14 +1055,12 @@ session(rfsv & a, rpcs & r, int xargc, char **xargv)
 
 #if HAVE_LIBREADLINE
 #if (READLINE_VERSION >= 402)
-#define NULL_COMPLETION_RETTYPE char*
-#define NULL_COMPLETION_RETVAL ""
-#define FUNCAST(f) (CPFunction *)f
+#define FUNCAST(f) f
+#define CPFUNCAST(f) f
 #define MATCHFUNCTION rl_completion_matches
 #else
-#define NULL_COMPLETION_RETTYPE int
-#define NULL_COMPLETION_RETVAL 0
 #define FUNCAST(f) (Function *)f
+#define CPFUNCAST(f) (CPPFunction *)f
 #define MATCHFUNCTION completion_matches
 #endif
 
@@ -987,7 +1069,7 @@ static char *all_commands[] = {
     "dir", "ls", "dircnt", "cd", "lcd", "get", "put", "mget", "mput",
     "del", "rm", "mkdir", "rmdir", "prompt", "bye", "cp", "volname",
     "ps", "kill", "killsave", "runrestore", "run", "machinfo",
-    "ownerinfo", NULL
+    "ownerinfo", "help", "setupinfo", NULL
 };
 
 static char *localfile_commands[] = {
@@ -1003,7 +1085,13 @@ static long maskAttr;
 static char cplPath[1024];
 
 static char*
-filename_generator(char *text, int state)
+filename_generator(
+#if (READLINE_VERSION >= 402)
+				   const char *text,
+#else
+				   char *text,
+#endif
+				   int state)
 {
     static int len;
     string tmp;
@@ -1040,7 +1128,13 @@ filename_generator(char *text, int state)
 }
 
 static char *
-command_generator(char *text, int state)
+command_generator(
+#if (READLINE_VERSION >= 402)
+		  const char *text,
+#else
+		  char *text,
+#endif
+		  int state)
 {
     static int idx, len;
     char *name;
@@ -1057,20 +1151,37 @@ command_generator(char *text, int state)
     return NULL;
 }
 
-static NULL_COMPLETION_RETTYPE
-null_completion() {
-    return NULL_COMPLETION_RETVAL;
+#if (READLINE_VERSION >= 402)
+static char * null_completion(const char *, int) {
+    return "";
 }
+#else
+static int null_completion() {
+    return 0;
+}
+#endif
 
 static char **
-do_completion(char *text, int start, int end)
+do_completion(const char *text, int start, int end)
 {
     char **matches = NULL;
 
     rl_completion_entry_function = FUNCAST(null_completion);
     rl_completion_append_character = ' ';
+#if (READLINE_VERSION >= 402)
+    rl_attempted_completion_over = 1;
+#endif
     if (start == 0)
-	matches = MATCHFUNCTION(text, cmdgen_ptr);
+	{
+#if HAVE_LIBREADLINE
+# if (READLINE_VERSION >= 402)
+	matches = MATCHFUNCTION(text, command_generator);
+# else
+	char* txt = (char*)text;
+	matches = MATCHFUNCTION(txt, (CPFunction*)command_generator);
+# endif
+#endif
+	}
     else {
 	int idx = 0;
 	char *name;
@@ -1098,7 +1209,16 @@ do_completion(char *text, int start, int end)
 		maskAttr = rfsv::PSI_A_DIR;
 	}
 
-	matches = MATCHFUNCTION(text, fnmgen_ptr);
+#if HAVE_LIBREADLINE
+# if (READLINE_VERSION >= 402)
+	matches = MATCHFUNCTION(text, filename_generator);
+# else
+	{
+	char* txt = (char*)text;
+	matches = MATCHFUNCTION(txt, (CPFunction*)filename_generator);
+	}
+# endif
+#endif
     }
     return matches;
 }
@@ -1110,9 +1230,14 @@ initReadline(void)
 #if HAVE_LIBREADLINE
     rl_readline_name = "plpftp";
     rl_completion_entry_function = FUNCAST(null_completion);
-    rl_attempted_completion_function = (CPPFunction *)do_completion;
+    rl_attempted_completion_function = CPFUNCAST(do_completion);
+#if (READLINE_VERSION < 402)
     rlcrap_setpointers(command_generator, filename_generator);
 #endif
+#if (READLINE_VERSION >= 402)
+    rl_basic_word_break_characters = " \t\n\"\\'`@><=;|&{(";
+#endif
+#endif
 }
 
 void ftp::
